﻿' =======================================================
' Simple Shutdown Scheduler (SSS)
' Copyright (c) 2008-2009 Alain Descotes (violent_ken)
' https://sourceforge.net/projects/sshutdown/
' =======================================================


' SSS is free software; you can redistribute it and/or modify
' it under the terms of the GNU General Public License as published by
' the Free Software Foundation; either version 3 of the License, or
' (at your option) any later version.
'
' SSS is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
' GNU General Public License for more details.
'
' You should have received a copy of the GNU General Public License
' along with SSS; if not, see http://www.gnu.org/licenses/.


Option Strict On

Imports System.Management
Imports System.Xml
Imports System.ComponentModel

<Serializable()> Public Class Scenario


    ' ========================================
    ' Private
    ' ========================================

    Private _isRemote As Boolean        ' Remote action ?
    Private _remote As TRemote          ' Remote target definition
    Private _action As EAction          ' Type of action
    Private _date As TDate              ' Date definition
    Private _enabled As Boolean         ' Scenario enabled ?
    Private _force As Boolean           ' Have to force ?
    Private _progPath As String         ' Program path if _action = [StartProgram]
    Private _autoRemove As Boolean      ' Has to be removed when executed ?
    Private _machine As String          ' Target machine
    Private _done As Boolean            ' Is scenario done ?



    ' ========================================
    ' Public structures/enums
    ' ========================================

    ' Type of action to do
    Public Enum EAction
        [StartProgram]
        [Shutdown]
        [Restart]
        [PowerOff]
        [Lock]
        [LogOff]
        [Hibernate]
        [Sleep]
    End Enum

    ' Type of date
    Public Enum EDate
        [Now]
        [Daily]
        [Weekly]
        [Monthly]
        [OneTime]
    End Enum

    ' Remote target
    Public Structure TRemote
        Public MachineName As String
        Public UserName As String
        Public Password As String
        Public Sub New(ByVal machine As String, ByVal user As String, ByVal pwd As System.Security.SecureString)
            MachineName = machine
            UserName = user
            Password = Misc.Encrypt(pwd)
        End Sub
        Public Overrides Function ToString() As String
            Return MachineName & ">" & UserName & ">" & Password.ToString
        End Function
    End Structure

    ' Date
    Public Structure TDate
        Public Type As EDate
        Public Time As TTime
        Public Sub New(ByVal typ As EDate, ByVal tim As TTime)
            Type = typ
            Time = tim
        End Sub
        Public Overrides Function ToString() As String
            Return Type.ToString & ", " & Time.ToString
        End Function
    End Structure

    ' Time
    Public Structure TTime
        Public Hour As Byte
        Public Minute As Byte
        Public Second As Byte
        Public Day As Byte
        Public Month As Byte
        Public Year As Integer
        Public Sub New(ByVal h As Byte, ByVal m As Byte, ByVal s As Byte, ByVal d As Byte, ByVal mo As Byte, ByVal y As Integer)
            Hour = h
            Minute = m
            Second = s
            Year = y
            Month = mo
            Day = d
        End Sub
        Public Sub New(ByVal h As Integer, ByVal m As Integer, ByVal s As Integer, ByVal d As Integer, ByVal mo As Integer, ByVal y As Integer)
            Hour = CByte(h)
            Minute = CByte(m)
            Second = CByte(s)
            Year = y
            Month = CByte(mo)
            Day = CByte(d)
        End Sub
        Public Sub New(ByRef d As Date)
            With d
                Hour = CByte(.Hour)
                Minute = CByte(.Minute)
                Second = CByte(.Second)
                Year = .Year
                Month = CByte(.Month)
                Day = CByte(.Day)
            End With
        End Sub
        Public Overrides Function ToString() As String
            Return New Date(Year, Month, Day, Hour, Minute, _
                            Second).ToString(My.Settings.DateTimeFormat)
        End Function
        Public Shared Operator >(ByVal t1 As TTime, _
                ByVal t2 As TTime) As Boolean
            Return (t1.Year > t2.Year _
                    OrElse (t1.Year = t2.Year And t1.Month > t2.Month) _
                    OrElse (t1.Year = t2.Year And t1.Month = t2.Month And t1.Day > t2.Day) _
                    OrElse (t1.Year = t2.Year And t1.Month = t2.Month And t1.Day = t2.Day _
                            And t1.Hour > t2.Hour) _
                    OrElse (t1.Year = t2.Year And t1.Month = t2.Month And t1.Day = t2.Day _
                            And t1.Hour = t2.Hour And t1.Minute > t2.Minute) _
                    OrElse (t1.Year = t2.Year And t1.Month = t2.Month And t1.Day = t2.Day _
                            And t1.Hour = t2.Hour And t1.Minute = t2.Minute _
                            And t1.Second > t2.Second))
        End Operator
        Public Shared Operator =(ByVal t1 As TTime, _
                ByVal t2 As TTime) As Boolean
            Return (t1.Hour = t2.Hour AndAlso t1.Minute = t2.Minute AndAlso t1.Second = t2.Second AndAlso t1.Year = t2.Year AndAlso t1.Month = t2.Month AndAlso t1.Day = t2.Day)
        End Operator
        Public Shared Operator <(ByVal t1 As TTime, _
                ByVal t2 As TTime) As Boolean
            Return Not (t1 > t2 Or t1 = t2)
        End Operator
        Public Shared Operator <>(ByVal t1 As TTime, _
                ByVal t2 As TTime) As Boolean
            Return Not (t1 = t2)
        End Operator
        Public Shared Operator <=(ByVal t1 As TTime, _
                ByVal t2 As TTime) As Boolean
            Return (t1 < t2 OrElse t1 = t2)
        End Operator
        Public Shared Operator >=(ByVal t1 As TTime, _
                ByVal t2 As TTime) As Boolean
            Return (t1 > t2 OrElse t1 = t2)
        End Operator
        Public Shared Operator -(ByVal t1 As TTime, _
                ByVal t2 As TTime) As Long
            Dim d1 As Date = New Date(t1.Year, t1.Month, t1.Day, t1.Hour, t1.Minute, t1.Second)
            Dim d2 As Date = New Date(t2.Year, t2.Month, t2.Day, t2.Hour, t2.Minute, t2.Second)
            Return d1.Subtract(d2).Ticks \ 10000000
        End Operator
        Public Function ReadyForDaily(ByVal t1 As TTime) As Boolean
            Return (t1.Hour = Hour AndAlso t1.Minute = Minute AndAlso t1.Second = Second)
        End Function
        Public Function ReadyForMothly(ByVal t1 As TTime) As Boolean
            Return (t1.Hour = Hour AndAlso t1.Minute = Minute AndAlso t1.Second = Second AndAlso t1.Day = Day)
        End Function
        Public Function ReadyForWeekly(ByVal t1 As TTime) As Boolean
            Dim i1 As DayOfWeek
            Dim i2 As DayOfWeek
            i1 = New Date(t1.Year, t1.Month, t1.Day, t1.Hour, t1.Minute, t1.Second).DayOfWeek
            i2 = New Date(Year, Month, Day, Hour, Minute, Second).DayOfWeek
            Return (t1.Hour = Hour AndAlso t1.Minute = Minute AndAlso t1.Second = Second AndAlso i1 = i2)
        End Function
        Public Function DailyDiff(ByVal t1 As TTime) As Long
            Dim d1 As Date = New Date(Year, Month, Day, t1.Hour, t1.Minute, t1.Second)
            Dim d2 As Date = New Date(Year, Month, Day, Hour, Minute, Second)
            Dim lng As Long = d2.Subtract(d1).Ticks \ 10000000
            If lng > 0 Then
                Return lng
            Else
                Return New Date().AddDays(1).Ticks \ 10000000 + lng
            End If
        End Function
        Public Function WeeklyDiff(ByVal t1 As TTime) As Long
            Dim d1 As Date = New Date(Year, Month, Day, t1.Hour, t1.Minute, t1.Second)
            Dim d2 As Date = New Date(Year, Month, Day, Hour, Minute, Second)
            Dim lng As Long = d2.Subtract(d1).Ticks \ 10000000
            If lng > 0 Then
                Return lng
            Else
                Return New Date().AddDays(7).Ticks \ 10000000 + lng
            End If
        End Function
        Public Function MonthlyDiff(ByVal t1 As TTime) As Long
            Dim d1 As Date = New Date(Year, Month, Day, t1.Hour, t1.Minute, t1.Second)
            Dim d2 As Date = New Date(Year, Month, Day, Hour, Minute, Second)
            Dim lng As Long = d2.Subtract(d1).Ticks \ 10000000
            If lng > 0 Then
                Return lng
            Else
                Return New Date().AddMonths(1).Ticks \ 10000000 + lng
            End If
        End Function
    End Structure




    ' ========================================
    ' Public properties
    ' ========================================

#Region "Properties"
    Public Property Done() As Boolean
        Get
            Return _done
        End Get
        Set(ByVal value As Boolean)
            _done = value
        End Set
    End Property
    Public Property AutoRemove() As Boolean
        Get
            Return _autoRemove
        End Get
        Set(ByVal value As Boolean)
            _autoRemove = value
        End Set
    End Property
    Public Property IsRemote() As Boolean
        Get
            Return _isRemote
        End Get
        Set(ByVal value As Boolean)
            _isRemote = value
        End Set
    End Property
    Public Property Enabled() As Boolean
        Get
            Return _enabled
        End Get
        Set(ByVal value As Boolean)
            _enabled = value
        End Set
    End Property
    Public Property Action() As EAction
        Get
            Return _action
        End Get
        Set(ByVal value As EAction)
            _action = value
        End Set
    End Property
    Public Property TheDate() As TDate
        Get
            Return _date
        End Get
        Set(ByVal value As TDate)
            _date = value
        End Set
    End Property
    Public Property Remote() As TRemote
        Get
            Return _remote
        End Get
        Set(ByVal value As TRemote)
            _remote = value
        End Set
    End Property
    Public Property Force() As Boolean
        Get
            Return _force
        End Get
        Set(ByVal value As Boolean)
            _force = value
        End Set
    End Property
    Public Property ProgramPath() As String
        Get
            Return _progPath
        End Get
        Set(ByVal value As String)
            _progPath = value
        End Set
    End Property
    Public ReadOnly Property MachineName() As String
        Get
            If _isRemote Then
                Return _remote.MachineName
            Else
                Return "Local"
            End If
        End Get
    End Property
#End Region


    ' ========================================
    ' Constructors
    ' ========================================
    Public Sub New(ByVal dat As TDate, ByVal force As Boolean, ByVal action As EAction, _
                   Optional ByVal path As String = Nothing)
        _date = dat
        _force = force
        _action = action
        _isRemote = False
        _progPath = path
    End Sub
    Public Sub New(ByVal dat As TDate, ByVal force As Boolean, ByVal action As EAction, _
                   ByVal remoteParam As TRemote, Optional ByVal path As String = Nothing)
        _date = dat
        _force = force
        _remote = remoteParam
        _progPath = path
        _action = action
        _isRemote = True
    End Sub
    Public Sub New()
        '
    End Sub



    ' ========================================
    ' Public functions
    ' ========================================

    ' Start scenario NOW !
    Public Sub Go()

        ' Only if enabled
        If Me.Enabled Then

            ' Set as 'done' if it's a "one time scenario"
            Me.Done = (Me.TheDate.Type = EDate.OneTime)

            Try
                ' Start scenario in a new thread pool
                System.Threading.ThreadPool.QueueUserWorkItem(New System.Threading.WaitCallback(AddressOf goAsync), _
                                                              Nothing)
            Catch ex As Exception
                Me.Enabled = False
                Me.Done = True
            End Try

        End If

    End Sub



    ' ========================================
    ' Private functions
    ' ========================================

#Region "Local shutdown functions"

    Private Function Hibernate(Optional ByVal force As Boolean = False) As Boolean
        Return Native.Api.NativeFunctions.SetSuspendState(True, force, False)
    End Function

    Private Function Sleep(Optional ByVal force As Boolean = False) As Boolean
        Return Native.Api.NativeFunctions.SetSuspendState(False, force, False)
    End Function

    Private Function Logoff(Optional ByVal force As Boolean = False) As Boolean
        If force Then
            Return Native.Api.NativeFunctions.ExitWindowsEx(Native.Api.NativeEnums.ExitWindowsFlags.Logoff Or Native.Api.NativeEnums.ExitWindowsFlags.Force, 0)
        Else
            Return Native.Api.NativeFunctions.ExitWindowsEx(Native.Api.NativeEnums.ExitWindowsFlags.Logoff, 0)
        End If
    End Function

    Private Function Lock() As Boolean
        Return Native.Api.NativeFunctions.LockWorkStation()
    End Function

    Private Function Shutdown(Optional ByVal force As Boolean = False) As Boolean
        If force Then
            Return Native.Api.NativeFunctions.ExitWindowsEx(Native.Api.NativeEnums.ExitWindowsFlags.Shutdown Or Native.Api.NativeEnums.ExitWindowsFlags.Force, 0)
        Else
            Return Native.Api.NativeFunctions.ExitWindowsEx(Native.Api.NativeEnums.ExitWindowsFlags.Shutdown, 0)
        End If
    End Function

    Private Function Restart(Optional ByVal force As Boolean = False) As Boolean
        If force Then
            Return Native.Api.NativeFunctions.ExitWindowsEx(Native.Api.NativeEnums.ExitWindowsFlags.Reboot Or Native.Api.NativeEnums.ExitWindowsFlags.Force, 0)
        Else
            Return Native.Api.NativeFunctions.ExitWindowsEx(Native.Api.NativeEnums.ExitWindowsFlags.Reboot, 0)
        End If
    End Function

    Private Function Poweroff(Optional ByVal force As Boolean = False) As Boolean
        If force Then
            Return Native.Api.NativeFunctions.ExitWindowsEx(Native.Api.NativeEnums.ExitWindowsFlags.Poweroff Or Native.Api.NativeEnums.ExitWindowsFlags.Force, 0)
        Else
            Return Native.Api.NativeFunctions.ExitWindowsEx(Native.Api.NativeEnums.ExitWindowsFlags.Poweroff, 0)
        End If
    End Function

#End Region

#Region "Remote functions"

    ' Execute a process remotely
    ' This may throw exceptions
    Private Shared Function ExecuteProcessRemote_TrowsException(ByVal TargetMachine As String, _
                                                                ByVal ProcessName As String, _
                                                                ByVal Paramaters As String, _
                                                                ByVal UserName As String, _
                                                                ByVal Password As System.Security.SecureString) As Integer
        Try
            Dim connOptions As New ConnectionOptions()
            With connOptions
                .EnablePrivileges = True
                .Impersonation = ImpersonationLevel.Impersonate
                .Username = UserName
                .SecurePassword = Password
            End With
            Dim manScope As New ManagementScope([String].Format("\\{0}\ROOT\CIMV2", TargetMachine), connOptions)
            manScope.Connect()
            Dim objectGetOptions As New ObjectGetOptions()
            Dim managementPath As New ManagementPath("Win32_Process")
            Dim processClass As New ManagementClass(manScope, managementPath, objectGetOptions)
            Dim inParams As ManagementBaseObject = processClass.GetMethodParameters("Create")
            inParams("CommandLine") = (ProcessName & " ") & Paramaters
            Dim outParams As ManagementBaseObject = processClass.InvokeMethod("Create", inParams, Nothing)
            Dim nReturnValue As Integer = Convert.ToInt32(outParams("returnValue").ToString())
            Select Case nReturnValue
                Case 0              ' Successful completion
                    If True Then
                        Return Convert.ToInt32(outParams("ProcessId"))
                    End If
                    Exit Select
                Case 2              ' Access denied
                    Throw New Win32Exception(5)
                    Exit Select
                Case 3              ' Insufficient Privilege
                    Throw New Win32Exception(8344)
                    Exit Select
                Case 8              ' Unknown failure
                    Throw New Win32Exception(13816)
                    Exit Select
                Case 9              ' Path Not Found
                    Throw New Win32Exception(3)
                    Exit Select
                Case 21             ' Invalid Parameter
                    Throw New Win32Exception(87)
                    Exit Select
                Case Else
                    Throw New Win32Exception(13816)
                    Exit Select
            End Select
        Catch e As Exception
            Throw e
        End Try
    End Function

#End Region

    ' Launch asynchronously the scenario
    Private Sub goAsync(ByVal asyncParam As Object)

        If Me.Enabled Then

            ' Disable the scenario if it's a 'one time scenario'
            If Me.TheDate.Type = EDate.OneTime Then Me.Enabled = False

            ' Autoremove ?
            If Me.AutoRemove AndAlso Me.TheDate.Type = EDate.OneTime Then

                ' Remove scenario NOW from list
                _scenarioList.Remove(Me)

                ' Save the list of scenarios
                _scenarioList.SaveData(XML_FILE)

                ' Refresh list of scenarios
                _frmMain.Invoke(New frmMain.UpdateListOfScenarios(AddressOf _
                                                                  _frmMain.impUpdateListOfScenarios))
            End If


            If Me.IsRemote Then

                ' It's a remote call, so we'll use WMI
                If Me.Action <> EAction.StartProgram Then

                    Dim _param As Native.Api.NativeEnums.WmiShutdownValues

                    ' Force action ?
                    If Me.Force Then
                        _param = _param Or Native.Api.NativeEnums.WmiShutdownValues.Force
                    End If

                    ' Type of action
                    Select Case _action
                        Case EAction.LogOff
                            _param = _param Or Native.Api.NativeEnums.WmiShutdownValues.LogOff
                        Case EAction.PowerOff
                            _param = _param Or Native.Api.NativeEnums.WmiShutdownValues.PowerOff
                        Case EAction.Restart
                            _param = _param Or Native.Api.NativeEnums.WmiShutdownValues.Reboot
                        Case EAction.Shutdown
                            _param = _param Or Native.Api.NativeEnums.WmiShutdownValues.Shutdown
                    End Select

                    ' Configure WMI
                    Dim __con As New ConnectionOptions
                    Dim obj(0) As Object
                    obj(0) = CObj(_param)
                    With __con
                        .Impersonation = ImpersonationLevel.Impersonate
                        .SecurePassword = Misc.DecryptSecure(Me.Remote.Password)
                        .Username = Me.Remote.UserName
                    End With

                    ' Launch the action
                    Try
                        Dim wmiSearcher As Management.ManagementObjectSearcher
                        wmiSearcher = New Management.ManagementObjectSearcher("SELECT * FROM Win32_OperatingSystem")
                        wmiSearcher.Scope = New Management.ManagementScope("\\" & _remote.MachineName & "\root\cimv2", __con)

                        For Each osObj As ManagementObject In wmiSearcher.Get
                            Call osObj.InvokeMethod("Win32Shutdown", obj)
                        Next
                    Catch ex As Exception
                        Call Misc.AddToLog(Me.Action, Me.MachineName, "Error : " & ex.Message)
                    End Try

                Else
                    ' Then we start a new process remotely
                    Try
                        Dim pid As Integer = ExecuteProcessRemote_TrowsException( _
                                            Me.Remote.MachineName, _
                                            Me.ProgramPath, _
                                            "", _
                                            Me.Remote.UserName, _
                                            Misc.DecryptSecure(Me.Remote.Password))
                        If pid = 0 Then
                            Call Misc.AddToLog(Me.Action, Me.MachineName, "Could not create process")
                        End If
                    Catch ex As Exception
                        Call Misc.AddToLog(Me.Action, Me.MachineName, "Error : " & ex.Message)
                    End Try
                End If

            Else
                ' Then this is a local shutdown
                Try
                    Select Case Me.Action
                        Case EAction.Hibernate
                            Call Me.Hibernate(Me.Force)
                        Case EAction.Lock
                            Call Me.Lock()
                        Case EAction.LogOff
                            Call Me.Logoff(Me.Force)
                        Case EAction.Restart
                            Call Me.Restart(Me.Force)
                        Case EAction.Shutdown
                            Call Me.Shutdown(Me.Force)
                        Case EAction.Sleep
                            Call Me.Sleep(Me.Force)
                        Case EAction.PowerOff
                            Call Me.Poweroff(Me.Force)
                        Case EAction.StartProgram
                            Process.Start(Me.ProgramPath)
                    End Select
                Catch ex As Exception
                    Call Misc.AddToLog(Me.Action, Me.MachineName, "Error : " & ex.Message)
                End Try

            End If

        End If

    End Sub

End Class
